@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -19,7 +19,7 @@
 
 """
 
-from nuitka.utils import Utils
+from nuitka.PythonVersions import python_version
 
 from .ConstantCodes import getConstantCode
 from .CoroutineCodes import getCoroutineObjectDeclCode
@@ -33,14 +33,10 @@ from .ErrorCodes import (
     getReleaseCode
 )
 from .GeneratorCodes import getGeneratorObjectDeclCode
+from .Helpers import generateExpressionCode
 from .Indentation import indented
+from .LabelCodes import getLabelCode
 from .ModuleCodes import getModuleAccessCode
-from .ParameterParsing import (
-    getDirectFunctionEntryPointIdentifier,
-    getParameterEntryPointIdentifier,
-    getParameterParsingCode,
-    getQuickEntryPointIdentifier
-)
 from .PythonAPICodes import getReferenceExportCode
 from .templates.CodeTemplatesFunction import (
     function_dict_setup,
@@ -54,8 +50,9 @@ from .templates.CodeTemplatesFunction import (
     template_make_function_with_context_template,
     template_make_function_without_context_template
 )
-from .TupleCodes import generateTupleCreationCode
+from .TupleCodes import getTupleCreationCode
 from .VariableCodes import (
+    getLocalVariableCodeType,
     getLocalVariableInitCode,
     getVariableCode,
     getVariableCodeName
@@ -120,14 +117,16 @@ def getFunctionMakerDecl(function_identifier, defaults_name, kw_defaults_name,
     }
 
 
-def getFunctionMakerCode(function_name, function_qualname, function_identifier,
-                         code_identifier, parameters, closure_variables,
-                         defaults_name, kw_defaults_name, annotations_name,
-                         function_doc, context):
-    # We really need this many parameters here. pylint: disable=R0913
+def getFunctionEntryPointIdentifier(function_identifier):
+    return "impl_" + function_identifier
 
-    # Functions have many details, that we express as variables
-    # pylint: disable=R0914
+
+def getFunctionMakerCode(function_name, function_qualname, function_identifier,
+                         code_identifier, closure_variables, defaults_name,
+                         kw_defaults_name, annotations_name, function_doc,
+                         context):
+    # We really need this many parameters here and functions have many details,
+    # that we express as variables, pylint: disable=R0914
     function_creation_args = _getFunctionCreationArgs(
         defaults_name     = defaults_name,
         kw_defaults_name  = kw_defaults_name,
@@ -135,7 +134,7 @@ def getFunctionMakerCode(function_name, function_qualname, function_identifier,
         closure_variables = closure_variables
     )
 
-    if Utils.python_version < 330 or function_qualname == function_name:
+    if python_version < 330 or function_qualname == function_name:
         function_qualname_obj = "NULL"
     else:
         function_qualname_obj = getConstantCode(
@@ -172,13 +171,9 @@ def getFunctionMakerCode(function_name, function_qualname, function_identifier,
             ),
             "function_qualname_obj"      : function_qualname_obj,
             "function_identifier"        : function_identifier,
-            "fparse_function_identifier" : getParameterEntryPointIdentifier(
+            "function_impl_identifier" : getFunctionEntryPointIdentifier(
                 function_identifier = function_identifier,
             ),
-            "dparse_function_identifier" : getQuickEntryPointIdentifier(
-                function_identifier = function_identifier,
-                parameters          = parameters
-            ),
             "function_creation_args"     : ", ".join(
                 function_creation_args
             ),
@@ -210,13 +205,9 @@ def getFunctionMakerCode(function_name, function_qualname, function_identifier,
             ),
             "function_qualname_obj"      : function_qualname_obj,
             "function_identifier"        : function_identifier,
-            "fparse_function_identifier" : getParameterEntryPointIdentifier(
+            "function_impl_identifier" : getFunctionEntryPointIdentifier(
                 function_identifier = function_identifier,
             ),
-            "dparse_function_identifier" : getQuickEntryPointIdentifier(
-                function_identifier = function_identifier,
-                parameters          = parameters
-            ),
             "function_creation_args"     : ", ".join(
                 function_creation_args
             ),
@@ -242,11 +233,17 @@ def getFunctionMakerCode(function_name, function_qualname, function_identifier,
     return result
 
 
-def generateFunctionCreationCode(to_name, function_body, code_object, defaults,
-                                 kw_defaults, annotations, defaults_first, emit,
-                                 context):
+def generateFunctionCreationCode(to_name, expression, emit, context):
     # This is about creating functions, which is detail ridden stuff,
     # pylint: disable=R0914
+
+    function_body  = expression.getFunctionRef().getFunctionBody()
+    code_object    = expression.getCodeObject()
+    defaults       = expression.getDefaults()
+    kw_defaults    = expression.getKwDefaults()
+    annotations    = expression.getAnnotations()
+    defaults_first = not expression.kw_defaults_before_defaults
+
     assert function_body.needsCreation(), function_body
 
     def handleKwDefaults():
@@ -256,7 +253,6 @@ def generateFunctionCreationCode(to_name, function_body, code_object, defaults,
             assert not kw_defaults.isExpressionConstantRef() or \
                    kw_defaults.getConstant() != {}, kw_defaults.getConstant()
 
-            from .CodeGeneration import generateExpressionCode
             generateExpressionCode(
                 to_name    = kw_defaults_name,
                 expression = kw_defaults,
@@ -272,7 +268,7 @@ def generateFunctionCreationCode(to_name, function_body, code_object, defaults,
         if defaults:
             defaults_name = context.allocateTempName("defaults")
 
-            generateTupleCreationCode(
+            getTupleCreationCode(
                 to_name  = defaults_name,
                 elements = defaults,
                 emit     = emit,
@@ -293,7 +289,6 @@ def generateFunctionCreationCode(to_name, function_body, code_object, defaults,
     if annotations:
         annotations_name = context.allocateTempName("annotations")
 
-        from .CodeGeneration import generateExpressionCode
         generateExpressionCode(
             to_name    = annotations_name,
             expression = annotations,
@@ -322,7 +317,6 @@ def generateFunctionCreationCode(to_name, function_body, code_object, defaults,
             function_qualname   = function_body.getFunctionQualname(),
             function_identifier = function_identifier,
             code_identifier     = code_identifier,
-            parameters          = function_body.getParameters(),
             closure_variables   = function_body.getClosureVariables(),
             defaults_name       = defaults_name,
             kw_defaults_name    = kw_defaults_name,
@@ -399,14 +393,24 @@ def getFunctionCreationCode(to_name, function_identifier, defaults_name,
 
 def getDirectFunctionCallCode(to_name, function_identifier, arg_names,
                               closure_variables, needs_check, emit, context):
-    function_identifier = getDirectFunctionEntryPointIdentifier(
+    function_identifier = getFunctionEntryPointIdentifier(
         function_identifier = function_identifier
     )
 
-    suffix_args = getClosureVariableProvisionCode(
-        context           = context,
-        closure_variables = closure_variables
-    )
+    suffix_args = []
+
+    for closure_variable in closure_variables:
+        variable_code_name, variable_c_type = getLocalVariableCodeType(
+            context  = context,
+            variable = closure_variable
+        )
+
+        if variable_c_type == "PyObject *":
+            suffix_args.append('&' + variable_code_name)
+        elif variable_c_type in ("PyCellObject *", "PyObject **"):
+            suffix_args.append(variable_code_name)
+        else:
+            assert False, variable_c_type
 
     # TODO: We ought to not assume references for direct calls, or make a
     # profile if an argument needs a reference at all. Most functions don't
@@ -418,15 +422,33 @@ def getDirectFunctionCallCode(to_name, function_identifier, arg_names,
         else:
             emit("Py_INCREF( %s );" % arg_name)
 
-    emit(
-        "%s = %s( %s );" % (
-            to_name,
-            function_identifier,
-            ", ".join(
-                arg_names + suffix_args
+
+    if arg_names:
+        emit(
+            """
+{
+    PyObject *dir_call_args[] = {%s};
+    %s = %s( dir_call_args%s%s );
+}""" % (
+                ", ".join(
+                    arg_names
+                ),
+                to_name,
+                function_identifier,
+                ", " if suffix_args else "",
+                ", ".join(suffix_args)
             )
         )
-    )
+    else:
+        emit(
+            "%s = %s( NULL%s%s );" % (
+                to_name,
+                function_identifier,
+                ", " if suffix_args else "",
+                ", ".join(suffix_args)
+            )
+        )
+
 
     # Arguments are owned to the called in direct function call.
     for arg_name in arg_names:
@@ -443,44 +465,24 @@ def getDirectFunctionCallCode(to_name, function_identifier, arg_names,
     context.addCleanupTempName(to_name)
 
 
-def getFunctionDirectClosureArgs(closure_variables):
-    result = []
-
-    for closure_variable in closure_variables:
-        if closure_variable.isSharedTechnically():
-            result.append(
-                "PyCellObject *%s" % (
-                    getVariableCodeName(
-                        in_context = True,
-                        variable   = closure_variable
-                    )
-                )
-            )
-        else:
-            # TODO: The reference is only needed for Python3, could make it
-            # version dependent.
-            result.append(
-                "PyObject *&%s" % (
-                    getVariableCodeName(
-                        in_context = True,
-                        variable   = closure_variable
-                    )
-                )
-            )
-
-    return result
-
-
-def getFunctionDirectDecl(function_identifier, closure_variables,
-                          parameter_variables, file_scope):
-
+def getFunctionDirectDecl(function_identifier, closure_variables, file_scope, context):
     parameter_objects_decl = [
-        "PyObject *_python_par_" + variable.getCodeName()
-        for variable in
-        parameter_variables
+        "PyObject **python_pars"
     ]
 
-    parameter_objects_decl += getFunctionDirectClosureArgs(closure_variables)
+    for closure_variable in closure_variables:
+        variable_code_name, variable_c_type = getLocalVariableCodeType(
+            context  = context,
+            variable = closure_variable
+        )
+
+        parameter_objects_decl.append(
+            "%s%s%s" % (
+                variable_c_type,
+                ' ' if variable_c_type[-1] in "*&" else ' ',
+                variable_code_name,
+            )
+        )
 
     result = template_function_direct_declaration % {
         "file_scope"           : file_scope,
@@ -491,25 +493,13 @@ def getFunctionDirectDecl(function_identifier, closure_variables,
     return result
 
 
-def getFunctionCode(context, function_name, function_identifier, parameters,
-                    closure_variables, user_variables, temp_variables,
-                    function_codes, function_doc, file_scope,
-                    needs_exception_exit):
-
-    # Many arguments, as we need much input transferred, pylint: disable=R0913
+def getFunctionCode(context, function_identifier, parameters, closure_variables,
+                    user_variables, temp_variables, function_codes, function_doc,
+                    file_scope, needs_exception_exit):
 
     # Functions have many details, that we express as variables, with many
     # branches to decide, pylint: disable=R0912,R0914
 
-    parameter_variables, entry_point_code, parameter_objects_decl = \
-      getParameterParsingCode(
-        function_identifier = function_identifier,
-        function_name       = function_name,
-        parameters          = parameters,
-        needs_creation      = context.isForCreatedFunction(),
-        context             = context,
-    )
-
     function_locals = []
 
     if context.hasLocalsDict():
@@ -518,18 +508,20 @@ def getFunctionCode(context, function_name, function_identifier, parameters,
     else:
         function_cleanup = ""
 
-    function_locals += [
-        getLocalVariableInitCode(
-            variable  = variable,
-            init_from = "_python_par_" + variable.getCodeName()
-        )
-        for variable in
-        parameter_variables
-    ]
+    if parameters is not None:
+        for count, variable in enumerate(parameters.getAllVariables()):
+            function_locals.append(
+                getLocalVariableInitCode(
+                    context   = context,
+                    variable  = variable,
+                    init_from = "python_pars[ %d ]" % count
+                )
+            )
 
     # User local variable initializations
     function_locals += [
         getLocalVariableInitCode(
+            context  = context,
             variable = variable,
         )
         for variable in
@@ -599,35 +591,52 @@ def getFunctionCode(context, function_name, function_identifier, parameters,
             }
         )
 
+    if context.isForCreatedFunction():
+        parameter_objects_decl = ["struct Nuitka_FunctionObject const *self"]
+    else:
+        parameter_objects_decl = []
+
+    parameter_objects_decl += [
+        "PyObject **python_pars"
+    ]
+
     if context.isForDirectCall():
-        parameter_objects_decl += getFunctionDirectClosureArgs(closure_variables)
+        for closure_variable in closure_variables:
+            variable_code_name, variable_c_type = getLocalVariableCodeType(
+                context  = context,
+                variable = closure_variable
+            )
+
+            parameter_objects_decl.append(
+                "%s%s%s" % (
+                    variable_c_type,
+                    ' ' if variable_c_type[-1] in "*&" else ' ',
+                    variable_code_name,
+                )
+            )
 
         result += function_direct_body_template % {
-            "file_scope"                   : file_scope,
-            "function_identifier"          : function_identifier,
-            "direct_call_arg_spec"         : ", ".join(
+            "file_scope"           : file_scope,
+            "function_identifier"  : function_identifier,
+            "direct_call_arg_spec" : ", ".join(
                 parameter_objects_decl
             ),
-            "function_locals"              : indented(function_locals),
-            "function_body"                : indented(function_codes),
-            "function_exit"                : function_exit
+            "function_locals"      : indented(function_locals),
+            "function_body"        : indented(function_codes),
+            "function_exit"        : function_exit
         }
     else:
         result += template_function_body % {
-            "function_identifier"          : function_identifier,
-            "parameter_objects_decl"       : ", ".join(parameter_objects_decl),
-            "function_locals"              : indented(function_locals),
-            "function_body"                : indented(function_codes),
-            "function_exit"                : function_exit
+            "function_identifier"    : function_identifier,
+            "parameter_objects_decl" : ", ".join(parameter_objects_decl),
+            "function_locals"        : indented(function_locals),
+            "function_body"          : indented(function_codes),
+            "function_exit"          : function_exit
         }
 
-    if context.isForCreatedFunction():
-        result += entry_point_code
-
     return result
 
 
-
 def getExportScopeCode(cross_module):
     if cross_module:
         return "NUITKA_CROSS_MODULE"
@@ -636,7 +645,7 @@ def getExportScopeCode(cross_module):
 
 
 
-def generateFunctionDeclCode(function_body):
+def generateFunctionDeclCode(function_body, context):
     if function_body.isExpressionGeneratorObjectBody():
         return getGeneratorObjectDeclCode(
             function_identifier = function_body.getCodeName(),
@@ -645,14 +654,102 @@ def generateFunctionDeclCode(function_body):
         return getCoroutineObjectDeclCode(
             function_identifier = function_body.getCodeName(),
         )
+    elif function_body.isExpressionClassBody():
+        return getFunctionDirectDecl(
+            function_identifier = function_body.getCodeName(),
+            closure_variables   = function_body.getClosureVariables(),
+            file_scope          = getExportScopeCode(
+                cross_module = False
+            ),
+            context             = context
+        )
     elif function_body.needsDirectCall():
         return getFunctionDirectDecl(
             function_identifier = function_body.getCodeName(),
             closure_variables   = function_body.getClosureVariables(),
-            parameter_variables = function_body.getParameters().getAllVariables(),
             file_scope          = getExportScopeCode(
                 cross_module = function_body.isCrossModuleUsed()
-            )
+            ),
+            context             = context
         )
     else:
         return None
+
+
+def generateFunctionCallCode(to_name, expression, emit, context):
+    assert expression.getFunction().isExpressionFunctionCreation()
+
+    function_body = expression.getFunction().getFunctionRef().getFunctionBody()
+    function_identifier = function_body.getCodeName()
+
+    argument_values = expression.getArgumentValues()
+
+    arg_names = []
+    for count, arg_value in enumerate(argument_values):
+        arg_name = context.allocateTempName("dircall_arg%d" % (count+1))
+
+        generateExpressionCode(
+            to_name    = arg_name,
+            expression = arg_value,
+            emit       = emit,
+            context    = context
+        )
+
+        arg_names.append(arg_name)
+
+    context.setCurrentSourceCodeReference(
+        expression.getCompatibleSourceReference()
+    )
+
+    getDirectFunctionCallCode(
+        to_name             = to_name,
+        function_identifier = function_identifier,
+        arg_names           = arg_names,
+        closure_variables   = function_body.getClosureVariables(),
+        needs_check         = expression.getFunction().getFunctionRef().\
+                                getFunctionBody().mayRaiseException(BaseException),
+        emit                = emit,
+        context             = context
+    )
+
+
+def generateFunctionOutlineCode(to_name, expression, emit, context):
+    assert expression.isExpressionOutlineBody()
+
+    # Need to set return target, to assign to_name from.
+    old_return_release_mode = context.getReturnReleaseMode()
+
+    return_target = context.allocateLabel("outline_result")
+    old_return_target = context.setReturnTarget(return_target)
+
+    return_value_name = context.allocateTempName("outline_return_value")
+    old_return_value_name = context.setReturnValueName(return_value_name)
+
+    from .CodeGeneration import generateStatementSequenceCode
+    generateStatementSequenceCode(
+        statement_sequence = expression.getBody(),
+        emit               = emit,
+        context            = context,
+        allow_none         = False
+    )
+
+    getMustNotGetHereCode(
+        reason  = "Return statement must have exited already.",
+        context = context,
+        emit    = emit
+    )
+
+    getLabelCode(return_target, emit)
+    emit(
+        "%s = %s;" % (
+            to_name,
+            return_value_name
+        )
+    )
+
+    context.addCleanupTempName(to_name)
+
+    # Restore previous "return" handling.
+    context.setReturnTarget(old_return_target)
+    context.setReturnReleaseMode(old_return_release_mode)
+    context.setReturnValueName(old_return_value_name)
