@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -20,7 +20,7 @@
 """
 
 from nuitka import Options, Variables
-from nuitka.utils import Utils
+from nuitka.PythonVersions import python_version
 
 from .ConstantCodes import getConstantCode
 from .Emission import SourceCodeCollector
@@ -29,10 +29,9 @@ from .ErrorCodes import (
     getErrorFormatExitBoolCode,
     getErrorFormatExitCode
 )
+from .Helpers import generateExpressionCode
 from .Indentation import indented
 from .templates.CodeTemplatesVariables import (
-    template_check_local,
-    template_check_shared,
     template_del_global_unclear,
     template_del_local_intolerant,
     template_del_local_known,
@@ -62,7 +61,33 @@ from .templates.CodeTemplatesVariables import (
 )
 
 
-def generateVariableDelCode(statement, emit, context):
+def generateAssignmentVariableCode(statement, emit, context):
+    variable_ref  = statement.getTargetVariableRef()
+    value         = statement.getAssignSource()
+
+    tmp_name = context.allocateTempName("assign_source")
+
+    generateExpressionCode(
+        expression = value,
+        to_name    = tmp_name,
+        emit       = emit,
+        context    = context
+    )
+
+    getVariableAssignmentCode(
+        tmp_name      = tmp_name,
+        variable      = variable_ref.getVariable(),
+        needs_release = statement.needsReleasePreviousValue(),
+        in_place      = statement.inplace_suspect,
+        emit          = emit,
+        context       = context
+    )
+
+    # Ownership of that reference must have been transfered.
+    assert not context.needsCleanup(tmp_name)
+
+
+def generateDelVariableCode(statement, emit, context):
     old_source_ref = context.setCurrentSourceCodeReference(
         statement.getSourceReference()
     )
@@ -78,6 +103,7 @@ def generateVariableDelCode(statement, emit, context):
 
     context.setCurrentSourceCodeReference(old_source_ref)
 
+
 def generateVariableReleaseCode(statement, emit, context):
     variable = statement.getVariable()
 
@@ -118,7 +144,7 @@ def getVariableCodeName(in_context, variable):
         return "var_" + variable.getCodeName()
 
 
-def _getLocalVariableCode(context, variable):
+def getLocalVariableCodeType(context, variable):
     # Now must be local or temporary variable, we return in each case-
     # pylint: disable=R0911
     user = context.getOwner()
@@ -130,32 +156,40 @@ def _getLocalVariableCode(context, variable):
             variable   = variable
         )
 
-        return result, variable.isSharedTechnically()
+        return (
+            result,
+            "PyCellObject *" if variable.isSharedTechnically() else "PyObject *"
+        )
     elif context.isForDirectCall():
         if user.isExpressionGeneratorObjectBody():
             closure_index = user.getClosureVariables().index(variable)
 
-            return "generator->m_closure[%d]" % closure_index, True
+            return "generator->m_closure[%d]" % closure_index, "PyCellObject *"
         elif user.isExpressionCoroutineObjectBody():
             closure_index = user.getClosureVariables().index(variable)
 
-            return "coroutine->m_closure[%d]" % closure_index, True
+            return "coroutine->m_closure[%d]" % closure_index, "PyCellObject *"
         else:
             result = getVariableCodeName(
                 in_context = True,
                 variable   = variable
             )
 
-            return result, variable.isSharedTechnically()
+            # TODO: The reference is only needed for Python3, could make it
+            # version dependent.
+            return (
+                result,
+                "PyCellObject *" if variable.isSharedTechnically() else "PyObject **",
+            )
     else:
         closure_index = user.getClosureVariables().index(variable)
 
         if user.isExpressionGeneratorObjectBody():
-            return "generator->m_closure[%d]" % closure_index, True
+            return "generator->m_closure[%d]" % closure_index, "PyCellObject *"
         elif user.isExpressionCoroutineObjectBody():
-            return "coroutine->m_closure[%d]" % closure_index, True
+            return "coroutine->m_closure[%d]" % closure_index, "PyCellObject *"
         else:
-            return "self->m_closure[%d]" % closure_index, True
+            return "self->m_closure[%d]" % closure_index, "PyCellObject *"
 
 
 def getVariableCode(context, variable):
@@ -166,47 +200,48 @@ def getVariableCode(context, variable):
             variable   = variable
         )
 
-    result, _is_cell = _getLocalVariableCode(context, variable)
-    return result
+    variable_code_name, _variable_c_type = getLocalVariableCodeType(context, variable)
+    return variable_code_name
 
 
 def getLocalVariableObjectAccessCode(context, variable):
-    assert variable.isLocalVariable()
+    variable_code_name, variable_c_type = getLocalVariableCodeType(context, variable)
 
-    code, is_cell = _getLocalVariableCode(context, variable)
-
-    if is_cell:
-        return code + "->ob_ref"
+    if variable_c_type == "PyCellObject *":
+        # TODO: Why not use PyCell_GET for readability.
+        return variable_code_name + "->ob_ref"
+    elif variable_c_type == "PyObject **":
+        return '*' + variable_code_name
+    elif variable_c_type == "PyObject *":
+        return variable_code_name
     else:
-        return code
+        assert False, variable_c_type
 
 
-def getLocalVariableInitCode(variable, init_from = None):
+def getLocalVariableInitCode(context, variable, init_from = None):
     assert not variable.isModuleVariable()
 
-    type_name = variable.getDeclarationTypeCode()
+    variable_code_name, variable_c_type = getLocalVariableCodeType(context, variable)
 
-    code_name = getVariableCodeName(
-        in_context = False,
-        variable   = variable
-    )
-
-    if variable.isSharedTechnically():
+    if variable_c_type == "PyCellObject *":
         # TODO: Single out "init_from" only user, so it becomes sure that we
         # get a reference transferred here in these cases.
         if init_from is not None:
             init_value = "PyCell_NEW1( %s )" % init_from
         else:
             init_value = "PyCell_EMPTY()"
-    else:
+    elif variable_c_type == "PyObject *":
         if init_from is None:
             init_from = "NULL"
 
         init_value = "%s" % init_from
+    else:
+        assert False, variable
 
-    return "%s%s = %s;" % (
-        type_name,
-        code_name,
+    return "%s%s%s = %s;" % (
+        variable_c_type,
+        ' ' if variable_c_type[-1] not in "*&" else "",
+        variable_code_name,
         init_value
     )
 
@@ -285,12 +320,14 @@ def getVariableAssignmentCode(context, emit, variable, tmp_name, needs_release,
         if ref_count:
             context.removeCleanupTempName(tmp_name)
     elif variable.isTempVariable():
-        if variable.isSharedTechnically():
+        _variable_code_name, variable_c_type = getLocalVariableCodeType(context, variable)
+
+        if variable_c_type == "PyCellObject *":
             if ref_count:
                 template = template_write_shared_unclear_ref0
             else:
                 template = template_write_shared_unclear_ref1
-        else:
+        elif variable_c_type in ("PyObject *", "PyObject **"):
             if ref_count:
                 if needs_release is False:
                     template = template_write_local_empty_ref0
@@ -305,10 +342,12 @@ def getVariableAssignmentCode(context, emit, variable, tmp_name, needs_release,
                     template = template_write_local_clear_ref1
                 else:
                     template = template_write_local_unclear_ref1
+        else:
+            assert False, variable_c_type
 
         emit(
             template % {
-                "identifier" : getVariableCode(context, variable),
+                "identifier" : getLocalVariableObjectAccessCode(context, variable),
                 "tmp_name"   : tmp_name
             }
         )
@@ -320,7 +359,7 @@ def getVariableAssignmentCode(context, emit, variable, tmp_name, needs_release,
 
 
 def getVariableAccessCode(to_name, variable, needs_check, emit, context):
-    # Many different cases, as this must be, pylint: disable=R0912
+    # Many different cases, as this must be, pylint: disable=R0912,R0915
 
     assert isinstance(variable, Variables.Variable), variable
 
@@ -337,7 +376,7 @@ def getVariableAccessCode(to_name, variable, needs_check, emit, context):
         )
 
         if needs_check:
-            if Utils.python_version < 340 and \
+            if python_version < 340 and \
                not context.isCompiledPythonModule() and \
                not context.getOwner().isExpressionClassBody():
                 error_message = "global name '%s' is not defined"
@@ -348,7 +387,8 @@ def getVariableAccessCode(to_name, variable, needs_check, emit, context):
                 check_name = to_name,
                 exception  = "PyExc_NameError",
                 args       = (
-                    error_message % variable.getName(),
+                    error_message,
+                    variable.getName()
                 ),
                 emit       = emit,
                 context    = context
@@ -382,7 +422,9 @@ def getVariableAccessCode(to_name, variable, needs_check, emit, context):
 
         return
     elif variable.isLocalVariable():
-        if variable.isSharedTechnically():
+        _variable_code_name, variable_c_type = getLocalVariableCodeType(context, variable)
+
+        if variable_c_type == "PyCellObject *":
             if not needs_check:
                 template = template_read_shared_unclear
             else:
@@ -411,9 +453,8 @@ def getVariableAccessCode(to_name, variable, needs_check, emit, context):
                     exception  = "PyExc_NameError",
                     args       = (
                         """\
-free variable '%s' referenced before assignment in enclosing scope""" % (
-                           variable.getName()
-                        ),
+free variable '%s' referenced before assignment in enclosing scope""",
+                        variable.getName()
                     ),
                     emit       = emit,
                     context    = context
@@ -424,9 +465,8 @@ free variable '%s' referenced before assignment in enclosing scope""" % (
                     exception  = "PyExc_UnboundLocalError",
                     args       = (
                         """\
-local variable '%s' referenced before assignment""" % (
-                           variable.getName()
-                        ),
+local variable '%s' referenced before assignment""",
+                        variable.getName()
                     ),
                     emit       = emit,
                     context    = context
@@ -436,7 +476,9 @@ local variable '%s' referenced before assignment""" % (
 
         return
     elif variable.isTempVariable():
-        if variable.isSharedTechnically():
+        _variable_code_name, variable_c_type = getLocalVariableCodeType(context, variable)
+
+        if variable_c_type == "PyCellObject *":
             template = template_read_shared_unclear
 
             emit(
@@ -452,9 +494,34 @@ local variable '%s' referenced before assignment""" % (
                     exception  = "PyExc_NameError",
                     args       = (
                         """\
-free variable '%s' referenced before assignment in enclosing scope""" % (
-                           variable.getName()
-                        ),
+free variable '%s' referenced before assignment in enclosing scope""",
+                        variable.getName()
+                    ),
+                    emit       = emit,
+                    context    = context
+                )
+            elif Options.isDebug():
+                emit("CHECK_OBJECT( %s );" % to_name)
+
+            return
+        elif variable_c_type in ("PyObject *", "PyObject **"):
+            template = template_read_local
+
+            emit(
+                template % {
+                    "tmp_name"   : to_name,
+                    "identifier" : getLocalVariableObjectAccessCode(context, variable)
+                }
+            )
+
+            if needs_check:
+                getErrorFormatExitCode(
+                    check_name = to_name,
+                    exception  = "PyExc_UnboundLocalError",
+                    args       = (
+                        """\
+local variable '%s' referenced before assignment""",
+                        variable.getName()
                     ),
                     emit       = emit,
                     context    = context
@@ -464,31 +531,7 @@ free variable '%s' referenced before assignment in enclosing scope""" % (
 
             return
         else:
-            template = template_read_local
-
-            emit(
-                template % {
-                    "tmp_name"   : to_name,
-                    "identifier" : getVariableCode(context, variable)
-                }
-            )
-
-            if needs_check:
-                getErrorFormatExitCode(
-                    check_name = to_name,
-                    exception  = "PyExc_UnboundLocalError",
-                    args       = ("""\
-local variable '%s' referenced before assignment""" % (
-                           variable.getName()
-                        ),
-                    ),
-                    emit       = emit,
-                    context    = context
-                )
-            elif Options.isDebug():
-                emit("CHECK_OBJECT( %s );" % to_name)
-
-            return
+            assert False, variable_c_type
 
     assert False, variable
 
@@ -662,25 +705,3 @@ def getVariableReleaseCode(variable, needs_check, emit, context):
             )
         }
     )
-
-
-def getVariableInitializedCheckCode(variable, context):
-    """ Check if a variable is initialized.
-
-        Returns a code expression that says "true" if it is.
-    """
-
-    if variable.isLocalVariable() or variable.isTempVariable():
-        if variable.isSharedTechnically():
-            template = template_check_shared
-        else:
-            template = template_check_local
-
-        return template % {
-            "identifier" : getVariableCode(
-                variable = variable,
-                context  = context
-            ),
-        }
-    else:
-        assert False, variable
