@@ -15,7 +15,7 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-""" Node the calls to the 'range' builtin.
+""" Node the calls to the 'range' built-in.
 
 This is a rather complex beast as it has many cases, is difficult to know if
 it's sizable enough to compute, and there are complex cases, where the bad
@@ -28,25 +28,9 @@ import math
 from nuitka.optimizations import BuiltinOptimization
 from nuitka.PythonVersions import python_version
 
-from .NodeBases import ExpressionBuiltinNoArgBase, ExpressionChildrenHavingBase
+from .NodeBases import ExpressionChildrenHavingBase
 from .NodeMakingHelpers import makeConstantReplacementNode
-
-
-class ExpressionBuiltinRange0(ExpressionBuiltinNoArgBase):
-    kind = "EXPRESSION_BUILTIN_RANGE0"
-
-    def __init__(self, source_ref):
-        ExpressionBuiltinNoArgBase.__init__(
-            self,
-            builtin_function = range,
-            source_ref       = source_ref
-        )
-
-    def mayHaveSideEffects(self):
-        return False
-
-    def mayBeNone(self):
-        return False
+from .shapes.BuiltinTypeShapes import ShapeTypeList
 
 
 class ExpressionBuiltinRangeBase(ExpressionChildrenHavingBase):
@@ -61,6 +45,9 @@ class ExpressionBuiltinRangeBase(ExpressionChildrenHavingBase):
             source_ref = source_ref
         )
 
+    def getTypeShape(self):
+        return ShapeTypeList
+
     def getTruthValue(self):
         length = self.getIterationLength()
 
@@ -106,17 +93,17 @@ class ExpressionBuiltinRangeBase(ExpressionChildrenHavingBase):
 
         return False
 
-    def computeBuiltinSpec(self, constraint_collection, given_values):
+    def computeBuiltinSpec(self, trace_collection, given_values):
         assert self.builtin_spec is not None, self
 
         if not self.builtin_spec.isCompileTimeComputable(given_values):
-            constraint_collection.onExceptionRaiseExit(BaseException)
+            trace_collection.onExceptionRaiseExit(BaseException)
 
             # TODO: Raise exception known step 0.
 
             return self, None, None
 
-        return constraint_collection.getCompileTimeComputationResult(
+        return trace_collection.getCompileTimeComputationResult(
             node        = self,
             computation = lambda : self.builtin_spec.simulateCall(
                 given_values
@@ -126,15 +113,13 @@ class ExpressionBuiltinRangeBase(ExpressionChildrenHavingBase):
             )
         )
 
-    def computeExpressionIter1(self, iter_node, constraint_collection):
-        # TODO: Support Python3 range objects too.
-        if python_version >= 300:
-            return iter_node, None, None
+    def computeExpressionIter1(self, iter_node, trace_collection):
+        assert python_version < 300
 
         iteration_length = self.getIterationLength()
 
         if iteration_length is not None and iteration_length > 256:
-            result = ExpressionBuiltinXrange(
+            result = makeExpressionBuiltinXrange(
                 low        = self.getLow(),
                 high       = self.getHigh(),
                 step       = self.getStep(),
@@ -153,6 +138,9 @@ class ExpressionBuiltinRangeBase(ExpressionChildrenHavingBase):
 
         return iter_node, None, None
 
+    def canPredictIterationValues(self):
+        return self.getIterationLength() is not None
+
     @staticmethod
     def getLow():
         return None
@@ -189,16 +177,14 @@ class ExpressionBuiltinRange1(ExpressionBuiltinRangeBase):
 
     getLow = ExpressionChildrenHavingBase.childGetter("low")
 
-    def computeExpression(self, constraint_collection):
-        # TODO: Support Python3 range objects too.
-        if python_version >= 300:
-            return self, None, None
+    def computeExpression(self, trace_collection):
+        assert python_version < 300
 
         low  = self.getLow()
 
         return self.computeBuiltinSpec(
-            constraint_collection = constraint_collection,
-            given_values          = (
+            trace_collection = trace_collection,
+            given_values     = (
                 low,
             )
         )
@@ -211,9 +197,6 @@ class ExpressionBuiltinRange1(ExpressionBuiltinRangeBase):
 
         return max(0, low)
 
-    def canPredictIterationValues(self):
-        return self.getIterationLength() is not None
-
     def getIterationValue(self, element_index):
         length = self.getIterationLength()
 
@@ -254,16 +237,15 @@ class ExpressionBuiltinRange2(ExpressionBuiltinRangeBase):
 
     builtin_spec = BuiltinOptimization.builtin_range_spec
 
-    def computeExpression(self, constraint_collection):
-        if python_version >= 300:
-            return self, None, None
+    def computeExpression(self, trace_collection):
+        assert python_version < 300
 
         low  = self.getLow()
         high = self.getHigh()
 
         return self.computeBuiltinSpec(
-            constraint_collection = constraint_collection,
-            given_values          = (
+            trace_collection = trace_collection,
+            given_values     = (
                 low,
                 high
             )
@@ -285,9 +267,6 @@ class ExpressionBuiltinRange2(ExpressionBuiltinRangeBase):
 
         return max(0, high - low)
 
-    def canPredictIterationValues(self):
-        return self.getIterationLength() is not None
-
     def getIterationValue(self, element_index):
         low  = self.getLow()
         high = self.getHigh()
@@ -342,17 +321,16 @@ class ExpressionBuiltinRange3(ExpressionBuiltinRangeBase):
 
     builtin_spec = BuiltinOptimization.builtin_range_spec
 
-    def computeExpression(self, constraint_collection):
-        if python_version >= 300:
-            return self, None, None
+    def computeExpression(self, trace_collection):
+        assert python_version < 300
 
         low  = self.getLow()
         high = self.getHigh()
         step = self.getStep()
 
         return self.computeBuiltinSpec(
-            constraint_collection = constraint_collection,
-            given_values          = (
+            trace_collection = trace_collection,
+            given_values     = (
                 low,
                 high,
                 step
@@ -430,13 +408,235 @@ class ExpressionBuiltinRange3(ExpressionBuiltinRangeBase):
         return count is None or count == self.getIterationLength()
 
 
-class ExpressionBuiltinXrange(ExpressionChildrenHavingBase):
-    kind = "EXPRESSION_BUILTIN_XRANGE"
+class ExpressionBuiltinXrangeBase(ExpressionChildrenHavingBase):
+    """ Base class for xrange nodes with 1/2/3 arguments. """
 
-    named_children = ("low", "high", "step")
+    builtin_spec = BuiltinOptimization.builtin_xrange_spec
+
+    def __init__(self, values, source_ref):
+        ExpressionChildrenHavingBase.__init__(
+            self,
+            values     = values,
+            source_ref = source_ref
+        )
+
+    def canPredictIterationValues(self):
+        return self.getIterationLength() is not None
+
+    def getTruthValue(self):
+        length = self.getIterationLength()
+
+        if length is None:
+            return None
+        else:
+            return length > 0
+
+    def mayHaveSideEffects(self):
+        for child in self.getVisitableNodes():
+            if child.mayHaveSideEffects():
+                return True
+
+            if child.getIntegerValue() is None:
+                return True
+
+        return False
+
+    def mayRaiseException(self, exception_type):
+        for child in self.getVisitableNodes():
+            if child.mayRaiseException(exception_type):
+                return True
+
+            # TODO: Should take exception_type value into account here.
+            if child.getIntegerValue() is None:
+                return True
+
+        step = self.getStep() # false alarm, pylint: disable=E1128
+
+        # A step of 0 will raise.
+        if step is not None and step.getIntegerValue() == 0:
+            return True
+
+        return False
+
+    def computeBuiltinSpec(self, trace_collection, given_values):
+        assert self.builtin_spec is not None, self
+
+        if not self.builtin_spec.isCompileTimeComputable(given_values):
+            trace_collection.onExceptionRaiseExit(BaseException)
+
+            # TODO: Raise exception known step 0.
+
+            return self, None, None
+
+        return trace_collection.getCompileTimeComputationResult(
+            node        = self,
+            computation = lambda : self.builtin_spec.simulateCall(
+                given_values
+            ),
+            description = "Built-in call to '%s' computed." % (
+                self.builtin_spec.getName()
+            )
+        )
+
+    def computeExpressionIter1(self, iter_node, trace_collection):
+        # No exception will be raised on xrange iteration.
+
+        return iter_node, None, None
+
+    @staticmethod
+    def getLow():
+        return None
+
+    @staticmethod
+    def getHigh():
+        return None
+
+    @staticmethod
+    def getStep():
+        return None
+
+    def mayBeNone(self):
+        return False
+
+
+class ExpressionBuiltinXrange1(ExpressionBuiltinXrangeBase):
+    kind = "EXPRESSION_BUILTIN_XRANGE1"
+
+    named_children = (
+        "low",
+    )
+
+    def __init__(self, low, source_ref):
+        ExpressionBuiltinXrangeBase.__init__(
+            self,
+            values     = {
+                "low" : low,
+            },
+            source_ref = source_ref
+        )
+
+    def computeExpression(self, trace_collection):
+        low = self.getLow()
+
+        return self.computeBuiltinSpec(
+            trace_collection = trace_collection,
+            given_values     = (
+                low,
+            )
+        )
+
+    getLow = ExpressionChildrenHavingBase.childGetter("low")
+
+    def getIterationLength(self):
+        low = self.getLow().getIntegerValue()
+
+        if low is None:
+            return None
+
+        return max(0, low)
+
+    def getIterationValue(self, element_index):
+        length = self.getIterationLength()
+
+        if length is None:
+            return None
+
+        if element_index > length:
+            return None
+
+        # TODO: Make sure to cast element_index to what CPython will give, for
+        # now a downcast will do.
+        return makeConstantReplacementNode(
+            constant = int(element_index),
+            node     = self
+        )
+
+
+class ExpressionBuiltinXrange2(ExpressionBuiltinXrangeBase):
+    kind = "EXPRESSION_BUILTIN_XRANGE2"
+
+    named_children = (
+        "low",
+        "high",
+    )
+
+    def __init__(self, low, high, source_ref):
+        ExpressionBuiltinXrangeBase.__init__(
+            self,
+            values     = {
+                "low"  : low,
+                "high" : high,
+            },
+            source_ref = source_ref
+        )
+
+    def computeExpression(self, trace_collection):
+        low  = self.getLow()
+        high = self.getHigh()
+
+        return self.computeBuiltinSpec(
+            trace_collection = trace_collection,
+            given_values     = (
+                low,
+                high
+            )
+        )
+
+    def getIterationLength(self):
+        low  = self.getLow()
+        high = self.getHigh()
+
+        low = low.getIntegerValue()
+
+        if low is None:
+            return None
+
+        high = high.getIntegerValue()
+
+        if high is None:
+            return None
+
+        return max(0, high - low)
+
+    def getIterationValue(self, element_index):
+        low  = self.getLow()
+        high = self.getHigh()
+
+        low = low.getIntegerValue()
+
+        if low is None:
+            return None
+
+        high = high.getIntegerValue()
+
+        if high is None:
+            return None
+
+        result = low + element_index
+
+        if result >= high:
+            return None
+        else:
+            return makeConstantReplacementNode(
+                constant = result,
+                node     = self
+            )
+
+    getLow  = ExpressionChildrenHavingBase.childGetter("low")
+    getHigh = ExpressionChildrenHavingBase.childGetter("high")
+
+
+class ExpressionBuiltinXrange3(ExpressionBuiltinXrangeBase):
+    kind = "EXPRESSION_BUILTIN_XRANGE3"
+
+    named_children = (
+        "low",
+        "high",
+        "step"
+    )
 
     def __init__(self, low, high, step, source_ref):
-        ExpressionChildrenHavingBase.__init__(
+        ExpressionBuiltinXrangeBase.__init__(
             self,
             values     = {
                 "low"  : low,
@@ -446,9 +646,105 @@ class ExpressionBuiltinXrange(ExpressionChildrenHavingBase):
             source_ref = source_ref
         )
 
-    def computeExpression(self, constraint_collection):
-        return self, None, None
+    def computeExpression(self, trace_collection):
+        low  = self.getLow()
+        high = self.getHigh()
+        step = self.getStep()
+
+        return self.computeBuiltinSpec(
+            trace_collection = trace_collection,
+            given_values     = (
+                low,
+                high,
+                step
+            )
+        )
 
     getLow  = ExpressionChildrenHavingBase.childGetter("low")
     getHigh = ExpressionChildrenHavingBase.childGetter("high")
     getStep = ExpressionChildrenHavingBase.childGetter("step")
+
+    def getIterationLength(self):
+        low  = self.getLow()
+        high = self.getHigh()
+        step = self.getStep()
+
+        low = low.getIntegerValue()
+
+        if low is None:
+            return None
+
+        high = high.getIntegerValue()
+
+        if high is None:
+            return None
+
+        step = step.getIntegerValue()
+
+        if step is None:
+            return None
+
+        # Give up on this, will raise ValueError.
+        if step == 0:
+            return None
+
+        if low < high:
+            if step < 0:
+                estimate = 0
+            else:
+                estimate = math.ceil(float(high - low) / step)
+        else:
+            if step > 0:
+                estimate = 0
+            else:
+                estimate = math.ceil(float(high - low) / step)
+
+        estimate = round(estimate)
+
+        assert estimate >= 0
+
+        return int(estimate)
+
+    def getIterationValue(self, element_index):
+        low  = self.getLow().getIntegerValue()
+
+        if low is None:
+            return None
+
+        high = self.getHigh().getIntegerValue()
+
+        if high is None:
+            return None
+
+        step = self.getStep().getIntegerValue()
+
+        result = low + step * element_index
+
+        if result >= high:
+            return None
+        else:
+            return makeConstantReplacementNode(
+                constant = result,
+                node     = self
+            )
+
+
+def makeExpressionBuiltinXrange(low, high, step, source_ref):
+    if high is None:
+        return ExpressionBuiltinXrange1(
+            low        = low,
+            source_ref = source_ref
+        )
+    elif step is None:
+        return ExpressionBuiltinXrange2(
+            low        = low,
+            high       = high,
+            source_ref = source_ref
+        )
+    else:
+        return ExpressionBuiltinXrange3(
+            low        = low,
+            high       = high,
+            step       = step,
+            source_ref = source_ref
+        )
